@@ -22,23 +22,17 @@ import logging
 import sys
 
 import httplib2
-from keystoneclient.auth.identity import v2 as v2_auth
-from keystoneclient.auth.identity import v3 as v3_auth
-from keystoneclient import discover
-from keystoneclient.openstack.common.apiclient import exceptions as ks_exc
-from keystoneclient import session as kssession
+from keystoneauth1.loading import session as kasession
 from oslo_utils import encodeutils
 import six
-import six.moves.urllib.parse as urlparse
-
 
 import ironicclient
 from ironicclient import client as iroclient
+from ironicclient.common import cliutils
 from ironicclient.common import http
 from ironicclient.common.i18n import _
 from ironicclient.common import utils
 from ironicclient import exc
-from ironicclient.openstack.common import cliutils
 
 
 LATEST_API_VERSION = ('1', 'latest')
@@ -46,46 +40,6 @@ LATEST_API_VERSION = ('1', 'latest')
 
 class IronicShell(object):
 
-    def _append_global_identity_args(self, parser):
-        # FIXME(dhu): these are global identity (Keystone) arguments which
-        # should be consistent and shared by all service clients. Therefore,
-        # they should be provided by python-keystoneclient. We will need to
-        # refactor this code once this functionality is avaible in
-        # python-keystoneclient.
-
-        # Register arguments needed for a Session
-        kssession.Session.register_cli_options(parser)
-
-        parser.add_argument('--os-user-domain-id',
-                            default=cliutils.env('OS_USER_DOMAIN_ID'),
-                            help='Defaults to env[OS_USER_DOMAIN_ID].')
-
-        parser.add_argument('--os-user-domain-name',
-                            default=cliutils.env('OS_USER_DOMAIN_NAME'),
-                            help='Defaults to env[OS_USER_DOMAIN_NAME].')
-
-        parser.add_argument('--os-project-id',
-                            default=cliutils.env('OS_PROJECT_ID'),
-                            help='Another way to specify tenant ID. '
-                                 'This option is mutually exclusive with '
-                                 ' --os-tenant-id. '
-                                 'Defaults to env[OS_PROJECT_ID].')
-
-        parser.add_argument('--os-project-name',
-                            default=cliutils.env('OS_PROJECT_NAME'),
-                            help='Another way to specify tenant name. '
-                                 'This option is mutually exclusive with '
-                                 ' --os-tenant-name. '
-                                 'Defaults to env[OS_PROJECT_NAME].')
-
-        parser.add_argument('--os-project-domain-id',
-                            default=cliutils.env('OS_PROJECT_DOMAIN_ID'),
-                            help='Defaults to env[OS_PROJECT_DOMAIN_ID].')
-
-        parser.add_argument('--os-project-domain-name',
-                            default=cliutils.env('OS_PROJECT_DOMAIN_NAME'),
-                            help='Defaults to env[OS_PROJECT_DOMAIN_NAME].')
-
     def get_base_parser(self):
         parser = argparse.ArgumentParser(
             prog='ironic',
@@ -96,13 +50,9 @@ class IronicShell(object):
             formatter_class=HelpFormatter,
         )
 
-        # FIXME(gyee): this method should come from python-keystoneclient.
-        # Will refactor this code once it is available.
-        # https://bugs.launchpad.net/python-keystoneclient/+bug/1332337
-
         # Register global Keystone args first so their defaults are respected.
         # See https://bugs.launchpad.net/python-ironicclient/+bug/1463581
-        self._append_global_identity_args(parser)
+        kasession.register_argparse_arguments(parser)
 
         # Global arguments
         parser.add_argument('-h', '--help',
@@ -119,6 +69,11 @@ class IronicShell(object):
                             action='store_true',
                             help='Defaults to env[IRONICCLIENT_DEBUG]')
 
+        parser.add_argument('--json',
+                            default=False,
+                            action='store_true',
+                            help='Print JSON response without formatting.')
+
         parser.add_argument('-v', '--verbose',
                             default=False, action="store_true",
                             help="Print more verbose output")
@@ -213,6 +168,7 @@ class IronicShell(object):
                             help=argparse.SUPPRESS)
 
         parser.add_argument('--os-endpoint',
+                            dest='ironic_url',
                             default=cliutils.env('OS_SERVICE_ENDPOINT'),
                             help='Specify an endpoint to use instead of '
                                  'retrieving one from the service catalog '
@@ -220,6 +176,7 @@ class IronicShell(object):
                                  'Defaults to env[OS_SERVICE_ENDPOINT].')
 
         parser.add_argument('--os_endpoint',
+                            dest='ironic_url',
                             help=argparse.SUPPRESS)
 
         parser.add_argument('--os-endpoint-type',
@@ -230,6 +187,36 @@ class IronicShell(object):
         parser.add_argument('--os_endpoint_type',
                             help=argparse.SUPPRESS)
 
+        parser.add_argument('--os-user-domain-id',
+                            default=cliutils.env('OS_USER_DOMAIN_ID'),
+                            help='Defaults to env[OS_USER_DOMAIN_ID].')
+
+        parser.add_argument('--os-user-domain-name',
+                            default=cliutils.env('OS_USER_DOMAIN_NAME'),
+                            help='Defaults to env[OS_USER_DOMAIN_NAME].')
+
+        parser.add_argument('--os-project-id',
+                            default=cliutils.env('OS_PROJECT_ID'),
+                            help='Another way to specify tenant ID. '
+                                 'This option is mutually exclusive with '
+                                 ' --os-tenant-id. '
+                                 'Defaults to env[OS_PROJECT_ID].')
+
+        parser.add_argument('--os-project-name',
+                            default=cliutils.env('OS_PROJECT_NAME'),
+                            help='Another way to specify tenant name. '
+                                 'This option is mutually exclusive with '
+                                 ' --os-tenant-name. '
+                                 'Defaults to env[OS_PROJECT_NAME].')
+
+        parser.add_argument('--os-project-domain-id',
+                            default=cliutils.env('OS_PROJECT_DOMAIN_ID'),
+                            help='Defaults to env[OS_PROJECT_DOMAIN_ID].')
+
+        parser.add_argument('--os-project-domain-name',
+                            default=cliutils.env('OS_PROJECT_DOMAIN_NAME'),
+                            help='Defaults to env[OS_PROJECT_DOMAIN_NAME].')
+
         parser.add_argument('--max-retries', type=int,
                             help='Maximum number of retries in case of '
                             'conflict error (HTTP 409). '
@@ -286,96 +273,6 @@ class IronicShell(object):
         commands.remove('bash-completion')
         print(' '.join(commands | options))
 
-    def _discover_auth_versions(self, session, auth_url):
-        # discover the API versions the server is supporting base on the
-        # given URL
-        v2_auth_url = None
-        v3_auth_url = None
-        try:
-            ks_discover = discover.Discover(session=session, auth_url=auth_url)
-            v2_auth_url = ks_discover.url_for('2.0')
-            v3_auth_url = ks_discover.url_for('3.0')
-        except ks_exc.ClientException:
-            # Identity service may not support discover API version.
-            # Let's try to figure out the API version from the original URL.
-            url_parts = urlparse.urlparse(auth_url)
-            (scheme, netloc, path, params, query, fragment) = url_parts
-            path = path.lower()
-            if path.startswith('/v3'):
-                v3_auth_url = auth_url
-            elif path.startswith('/v2'):
-                v2_auth_url = auth_url
-            else:
-                # not enough information to determine the auth version
-                msg = _('Unable to determine the Keystone version '
-                        'to authenticate with using the given '
-                        'auth_url. Identity service may not support API '
-                        'version discovery. Please provide a versioned '
-                        'auth_url instead. %s') % auth_url
-                raise exc.CommandError(msg)
-
-        return (v2_auth_url, v3_auth_url)
-
-    def _get_keystone_v3_auth(self, v3_auth_url, **kwargs):
-        auth_token = kwargs.pop('auth_token', None)
-        if auth_token:
-            return v3_auth.Token(v3_auth_url, auth_token)
-        else:
-            return v3_auth.Password(v3_auth_url, **kwargs)
-
-    def _get_keystone_v2_auth(self, v2_auth_url, **kwargs):
-        auth_token = kwargs.pop('auth_token', None)
-        if auth_token:
-            return v2_auth.Token(v2_auth_url, auth_token,
-                                 tenant_id=kwargs.pop('project_id', None),
-                                 tenant_name=kwargs.pop('project_name', None))
-        else:
-            return v2_auth.Password(
-                v2_auth_url,
-                username=kwargs.pop('username', None),
-                password=kwargs.pop('password', None),
-                tenant_id=kwargs.pop('project_id', None),
-                tenant_name=kwargs.pop('project_name', None))
-
-    def _get_keystone_auth(self, session, auth_url, **kwargs):
-        # FIXME(dhu): this code should come from keystoneclient
-
-        # discover the supported keystone versions using the given url
-        (v2_auth_url, v3_auth_url) = self._discover_auth_versions(
-            session=session,
-            auth_url=auth_url)
-
-        # Determine which authentication plugin to use. First inspect the
-        # auth_url to see the supported version. If both v3 and v2 are
-        # supported, then use the highest version if possible.
-        auth = None
-        if v3_auth_url and v2_auth_url:
-            user_domain_name = kwargs.get('user_domain_name', None)
-            user_domain_id = kwargs.get('user_domain_id', None)
-            project_domain_name = kwargs.get('project_domain_name', None)
-            project_domain_id = kwargs.get('project_domain_id', None)
-
-            # support both v2 and v3 auth. Use v3 if domain information is
-            # provided.
-            if (user_domain_name or user_domain_id or project_domain_name or
-                    project_domain_id):
-                auth = self._get_keystone_v3_auth(v3_auth_url, **kwargs)
-            else:
-                auth = self._get_keystone_v2_auth(v2_auth_url, **kwargs)
-        elif v3_auth_url:
-            # support only v3
-            auth = self._get_keystone_v3_auth(v3_auth_url, **kwargs)
-        elif v2_auth_url:
-            # support only v2
-            auth = self._get_keystone_v2_auth(v2_auth_url, **kwargs)
-        else:
-            msg = _('Unable to determine the Keystone version '
-                    'to authenticate with using the given '
-                    'auth_url.')
-            raise exc.CommandError(msg)
-
-        return auth
-
     def _check_version(self, api_version):
         if api_version == 'latest':
             return LATEST_API_VERSION
@@ -434,7 +331,7 @@ class IronicShell(object):
             self.do_bash_completion()
             return 0
 
-        if not (args.os_auth_token and (args.ironic_url or args.os_endpoint)):
+        if not (args.os_auth_token and (args.ironic_url or args.os_auth_url)):
             if not args.os_username:
                 raise exc.CommandError(_("You must provide a username via "
                                          "either --os-username or via "
@@ -470,67 +367,26 @@ class IronicShell(object):
                                          "either --os-auth-url or via "
                                          "env[OS_AUTH_URL]"))
 
-        endpoint = args.ironic_url or args.os_endpoint
-        service_type = args.os_service_type or 'baremetal'
-        project_id = args.os_project_id or args.os_tenant_id
-        project_name = args.os_project_name or args.os_tenant_name
-
-        if (args.os_auth_token and (args.ironic_url or args.os_endpoint)):
-            kwargs = {
-                'token': args.os_auth_token,
-                'insecure': args.insecure,
-                'timeout': args.timeout,
-                'ca_file': args.os_cacert,
-                'cert_file': args.os_cert,
-                'key_file': args.os_key,
-                'auth_ref': None,
-            }
-        elif (args.os_username and
-              args.os_password and
-              args.os_auth_url and
-              (project_id or project_name)):
-
-            keystone_session = kssession.Session.load_from_cli_options(args)
-
-            kwargs = {
-                'username': args.os_username,
-                'user_domain_id': args.os_user_domain_id,
-                'user_domain_name': args.os_user_domain_name,
-                'password': args.os_password,
-                'auth_token': args.os_auth_token,
-                'project_id': project_id,
-                'project_name': project_name,
-                'project_domain_id': args.os_project_domain_id,
-                'project_domain_name': args.os_project_domain_name,
-            }
-            keystone_auth = self._get_keystone_auth(keystone_session,
-                                                    args.os_auth_url,
-                                                    **kwargs)
-            if not endpoint:
-                svc_type = args.os_service_type
-                region_name = args.os_region_name
-                endpoint = keystone_auth.get_endpoint(keystone_session,
-                                                      service_type=svc_type,
-                                                      region_name=region_name)
-
-            endpoint_type = args.os_endpoint_type or 'publicURL'
-            kwargs = {
-                'session': keystone_session,
-                'auth': keystone_auth,
-                'service_type': service_type,
-                'endpoint_type': endpoint_type,
-                'region_name': args.os_region_name,
-            }
-        kwargs['os_ironic_api_version'] = os_ironic_api_version
         if args.max_retries < 0:
             raise exc.CommandError(_("You must provide value >= 0 for "
                                      "--max-retries"))
         if args.retry_interval < 1:
             raise exc.CommandError(_("You must provide value >= 1 for "
                                      "--retry-interval"))
-        kwargs['max_retries'] = args.max_retries
-        kwargs['retry_interval'] = args.retry_interval
-        client = iroclient.Client(api_major_version, endpoint, **kwargs)
+        client_args = (
+            'os_auth_token', 'ironic_url', 'os_username', 'os_password',
+            'os_auth_url', 'os_project_id', 'os_project_name', 'os_tenant_id',
+            'os_tenant_name', 'os_region_name', 'os_user_domain_id',
+            'os_user_domain_name', 'os_project_domain_id',
+            'os_project_domain_name', 'os_service_type', 'os_endpoint_type',
+            'os_cacert', 'os_cert', 'os_key', 'max_retries', 'retry_interval',
+            'timeout', 'insecure'
+        )
+        kwargs = {}
+        for key in client_args:
+            kwargs[key] = getattr(args, key)
+        kwargs['os_ironic_api_version'] = os_ironic_api_version
+        client = iroclient.get_client(api_major_version, **kwargs)
 
         try:
             args.func(client, args)
