@@ -19,6 +19,8 @@ os_auth_url=$OS_AUTH_URL
 os_username=$OS_USERNAME
 os_password=$OS_PASSWORD
 os_tenant_name=$OS_TENANT_NAME
+os_user_domain_id=$OS_USER_DOMAIN_ID
+os_project_domain_id=$OS_PROJECT_DOMAIN_ID
 os_service_type=baremetal
 os_endpoint_type=public
 END
